Re:Experience converting a mathematical software package to ...
by ttoinou on Tuesday July 01, 2025 @02:51PM
(#44436956)
40 seconds for 1M lines seems super fast, do you have a fast computer and/or did you spend a lot of time optimizing the compilation pipeline ?
Re:Experience converting a mathematical software package to ...
by vblanco on Tuesday July 01, 2025 @03:05PM
(#44437056)
The modern cryengine compiles very fast. Their trick is that they have architected everything to go through interfaces that are on very thin headers, and thus their headers end very light and they dont compile the class properties over and over. But its a shame we need to do tricks like this for compile speed as they harm runtime performance.
Re:Experience converting a mathematical software package to ...
by ttoinou on Tuesday July 01, 2025 @03:26PM
(#44437223)
Why does it ruin runtime performance ? The code should be almost the same
Re:Experience converting a mathematical software package to ...
by vblanco on Tuesday July 01, 2025 @03:34PM
(#44437281)
Because you now need to go through virtual calls on functions that dont really need to be virtual, which means the possible cache miss from loading the virtual function from vtable, and then the impossibility of them being inlined. For example they have a ITexture interface with a function like virtual GetSize(). If it wasnt all through virtuals, that size would just be a vec2 in the class and then its a simple load that gets inlined.
Re:Experience converting a mathematical software package to ...
by ttoinou on Tuesday July 01, 2025 @03:37PM
(#44437305)
Ah yes this kind of interface ok indeed this doesn't seem like a useful layer when running the program. Maybe the compilers could optimize this though
Re:Experience converting a mathematical software package to ...
by jeremiahar on Tuesday July 01, 2025 @08:45PM
(#44439278)
In my experience, as long as there's only a single implementation, devirtualization works well, and can even inline the functions. But you need to pass something along the lines of "-fwhole-program-vtables -fstrict-vtable-pointer" + LTO. Of course the vtable pointer is still present in the object. So I personally only use the aforementioned "thin headers" at a system level (IRenderer), rather than for each individual object (ITexture).
Re:Experience converting a mathematical software package to ...
by drysine on Wednesday July 02, 2025 @05:33AM
(#44441694)
They can sometimes
https://quuxplusone.github.io/blog/2021/02/15/devirtualizati...
Re:Experience converting a mathematical software package to ...
by pjmlp on Wednesday July 02, 2025 @04:48AM
(#44441425)
At least on clang with LTO, with bitcode variant, that should be possible to devirtualize, assuming most of those interfaces only have a single implementation.
Re:Experience converting a mathematical software package to ...
by barchar on Tuesday July 01, 2025 @08:56PM
(#44439322)
In addition to what everyone else has said it also makes it difficult to allocate the type on the stack. Even if you do allow it you'll at least need a probe.
Re:Experience converting a mathematical software package to ...
by w4rh4wk5 on Tuesday July 01, 2025 @04:36PM
(#44437789)
We didn't create this code base ourselves, we are just working with it. I'd assume the original developers payed attention to compile times during development and introduced forward declarations whenever things got out of hand.
My computer is fast, AMD Ryzen 9 7950X, code is stored on an NVMe SSD. But there certainly are projects with fewer lines of code that take substantially longer to compile.
We are commonly working with games that come with a custom engine and tooling. Compiling everything from scratch (around 1M lines of modern C++ code) takes about 30-40 seconds on my desktop. Rebuilding 1 source file + linking comes in typically under 2 seconds (w/o LTO). We might get this even lower by introducing unity builds, but there's no need for that right now.